/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import ij.*;
import ij.plugin.frame.*;

public class CatalogActions extends PlugInFrame implements ActionListener{

	Panel panMaster;
	Panel panEdit;
	Panel panView;
	Panel panAction;
	Button[] btEdit;
	Button[] btView;
	Button[] btActn;
	int btH = 30;
	int btW = 100;
	int insetT = 40;
	int insetB = 10;
	int insetL = 10;
	int insetR = 10;
	int btRows = 35;
	int btCols = 3;
	
	Catalog_ cat;
	Button as = new Button("Add Sample");
	Button am = new Button("Add Muscle");
	Button ae = new Button("Add Experiment");
	Button aif = new Button("Add ImageField");
	Button asi = new Button("Add Subimage");
	
	public CatalogActions(String title) {
		super(title);
		this.setLayout(null);
		this.setSize(btCols*btW+insetR+insetL,btRows*btH+insetT+insetB);
		panMaster = makeMasterPanel();
		add(panMaster);
		labelButtons();
		
		show();
	}
	
	public void setCatalog(Catalog_ cat) {
		this.cat = cat;
	}
	
	public Panel makeMasterPanel() {
		Panel mp = new Panel();
		mp.setLayout(null);
		mp.setBackground(Color.gray);
		mp.setBounds(insetL,insetT,btCols*btW,btRows*btH);
		panEdit = makeEditPanel();
		mp.add(panEdit);
		panView = makeViewPanel();
		mp.add(panView);
		panAction = makeActionPanel();
		mp.add(panAction);
		return mp;
	}
	
	public Panel makeEditPanel() {
		Panel ep = new Panel();
		ep.setLayout(null);
		ep.setBackground(Color.gray);
		ep.setBounds(0,0,btW,btRows*btH);
		int nBtEdit = this.btRows;
		btEdit = new Button [nBtEdit];
		for (int i=0; i<nBtEdit; i++) {
			btEdit[i] = new Button("");
			btEdit[i].setBounds(0,btH*i,btW,btH);
			btEdit[i].setBackground(Color.lightGray);
			btEdit[i].addActionListener(this);
			ep.add(btEdit[i]);
		}
		return ep;
	}
	
	public Panel makeViewPanel() {
		Panel vp = new Panel();
		vp.setLayout(null);
		vp.setBackground(Color.gray);
		vp.setBounds(btW,0,btW,btRows*btH);
		int nBtView = this.btRows;
		btView = new Button [nBtView];
		for (int i=0; i<nBtView; i++) {
			btView[i] = new Button("");
			btView[i].setBounds(0,btH*i,btW,btH);
			btView[i].setBackground(Color.green);
			btView[i].addActionListener(this);
			vp.add(btView[i]);
		}
		return vp;
	}
	
	public Panel makeActionPanel() {
		Panel ap = new Panel();
		ap.setLayout(null);
		ap.setBackground(Color.gray);
		ap.setBounds(2*btW,0,btW,btRows*btH);
		int nBtActn = this.btRows;
		btActn = new Button [nBtActn];
		for (int i=0; i<nBtActn; i++) {
			btActn[i] = new Button("");
			btActn[i].setBounds(0,btH*i,btW,btH);
			btActn[i].setBackground(Color.red);
			btActn[i].addActionListener(this);
			ap.add(btActn[i]);
		}
		return ap;
	}
	
	public void labelButtons() {
		btEdit[0].setLabel("Open RabbitSampleList");
		btEdit[1].setLabel("Save RabbitSampleList");
		btEdit[2].setLabel("Add RabbitSample");
		btEdit[3].setLabel("Edit RabbitSampleProps");
		btEdit[4].setLabel("Delete RabbitSampleProps");
		btEdit[5].setLabel("Open MuscleSampleList");
		btEdit[6].setLabel("Save MuscleSampleList");
		btEdit[7].setLabel("Add MuscleSample");
		btEdit[8].setLabel("Edit MuscleSample");
		btEdit[9].setLabel("Delete MuscleSample");
		btEdit[10].setLabel("Open MuscleTypeList");
		btEdit[11].setLabel("Save MuscleTypeList");
		btEdit[12].setLabel("Add MuscleType");
		btEdit[13].setLabel("Edit MuscleTypeProps");
		btEdit[14].setLabel("Delete MuscleTypeProps");
		btEdit[15].setLabel("Open ExperimentList");
		btEdit[16].setLabel("Save ExperimentList");
		btEdit[17].setLabel("Add Experiment");
		btEdit[18].setLabel("Edit Experiment");
		btEdit[19].setLabel("Delete Experiment");
		btEdit[20].setLabel("Open ImageFieldList");
		btEdit[21].setLabel("Save ImageFieldList");
		btEdit[22].setLabel("Add ImageField");
		btEdit[23].setLabel("Edit ImageFieldProps");
		btEdit[24].setLabel("View ImageFieldProps");
		btEdit[25].setLabel("Add SubImage To ImageField");
		btEdit[26].setLabel("View ProcedureList");
		btEdit[27].setLabel("Add Procedure To Experiment");
		btEdit[28].setLabel("Edit ProcedureProps in Experiment");
		btEdit[29].setLabel("Open ProbeTypeList");
		btEdit[30].setLabel("Save ProbeTypeList");
		btEdit[31].setLabel("Add ProbeType");
		btEdit[32].setLabel("Edit ProbeTypeProps");
		btEdit[33].setLabel("Import Measurement");
		btEdit[34].setLabel("Import Results");

		btView[0].setLabel("Sample");
		btView[1].setLabel("MuscleTypes");
		btView[2].setLabel("MuscleSamples");
		btView[3].setLabel("Experiments");
		btView[4].setLabel("Procedures");
		btView[5].setLabel("ProbeTypes");
		btView[6].setLabel("ImageFields");
		btView[7].setLabel("Measurements");

		btActn[0].setLabel("Open DB");
		btActn[1].setLabel("New DB");
		btActn[2].setLabel("Save DB");
		btActn[3].setLabel("DV SubImage");
		btActn[4].setLabel("DVi SubImage");
		btActn[5].setLabel("TIF SubImage");
		btActn[6].setLabel("Load Line");
		btActn[7].setLabel("LSWindow");
		btActn[8].setLabel("LSImage");
		btActn[9].setLabel("DDWindow");
		btActn[10].setLabel("ViewGraph");
	}
	
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		System.out.println(cmd);
		if (cmd.equals("Open RabbitSampleList")) {
			cat.openRabbitList();
		}
		else if (cmd.equals("Add RabbitSample")) {
			cat.addRabbit();
		}
		else if (cmd.equals("+Muscle")) {
			cat.addMuscle();
		}
		else if (cmd.equals("Add MuscleType")) {
			cat.addMuscleType();
		}
		else if (cmd.equals("Open MuscleTypeList")) {
			cat.openMuscleTypeList();
		}
		else if (cmd.equals("+Experiment")) {
			System.out.println("Adding Experiment");
		}
		else if (cmd.equals("+ImageField")) {
			System.out.println("Adding ImageField");
		}
		else if (cmd.equals("+SubImage")) {
			System.out.println("Adding SubImage");
		}
	}
}
