/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.ImageCanvas;
import ij.io.SaveDialog;
import ij.measure.ResultsTable;
import ij.plugin.filter.Analyzer;
import ij.plugin.frame.PlugInFrame;
import ij.plugin.frame.Recorder;
import ij.text.TextPanel;
import ij.text.TextWindow;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

//import ExcelInterface.*;

public class DDwindow extends PlugInFrame implements ActionListener, ItemListener, MouseListener, MouseMotionListener {

//	public static final double PLOT_W_PERPIXEL = Prefs.get("LSDDecon.pwpp",4.0);
	public ImageCanvas[] cans = new ImageCanvas[4];
	
	public DDplotter ddp;
	
	
	// Components of the window
	private Panel panMaster;
	private Panel panProf, panTFplot;//, panProfR, panTFAR;
	private Panel panZparams;//, panTF, panInfo, panPF;
	private Panel panTFparams;
	private int pInfowidth = 150;//, pInfoheight = 600;
	public TextField[] tfModelParams;
	private TextField tfNumZ;
	private TextField[] tfZp, tfZw;
	public LField lfImage, lfPFID, lfDate, lfSample, lfFluor, lfExpT;
	private Choice chModel;
	public TextArea taInfo;
	private Label labPerArea, labCorrel, labRsq;
	private CLabel labError;
	public Checkbox[] cbModelStates;
//	private Checkbox cbfxPSFw, cbfxTFlen, cbfxTFuni, cbfxTFpend; 
	public Checkbox cbfxSL, cbfxRW;
	private Checkbox cbfxZp1, cbfxZw1;
	private Checkbox[] cbfxZp, cbfxZw;
	private Button butNormArea, butFitPF, butExport, butViewer;
	
	
//	public DDProcessor dd;
//	public DDinfo ddi;
//	public LSDDecon_Manager manager;
	DDCurveFitter fitter;
	public boolean complete = false;
	
	private boolean isZdistPtSelected = false;
	private int selPtInd = -1;
    private static final int TEXT_GAP = 10;
	private static boolean centerOnScreen;
	
	DDwindow (DDplotter ddp) {
		super("DDecon Analysis of "+ddp.ls.lsName);
		this.ddp = ddp;
		
		cans[0] = new ImageCanvas(ddp.modelProfileImp);
		cans[1] = new ImageCanvas(ddp.modelResidualImp);
		cans[2] = new ImageCanvas(ddp.repeatProfileImp);
		cans[3] = new ImageCanvas(ddp.repeatResidualImp);

		if (ddp.repeatResidualPixels == null) IJ.write("DDW.<init>: residuals are null");
		makeWindow();
		
		// Start initialize()
		chModel.select(ddp.rm.model.getTitle());
		lfImage.setText(ddp.ls.lsName);
		lfPFID.setText(ddp.ls.lsName);
		lfSample.setText(Prefs.get("LSDDecon.sample",""));
		lfFluor.setText(Prefs.get("LSDDecon.fluor",""));
		lfDate.setText(Prefs.get("LSDDecon.date",""));
		lfExpT.setText(Prefs.get("LSDDecon.exposure",""));
		
		boolean[] pStates = ddp.rm.model.getParamStates();
		for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
			cbModelStates[i].setState(pStates[i]);
			cbModelStates[i].addItemListener(this);
		}
		// End intitialize()
		
		show();
		WindowManager.addWindow(this);
	}
	
/*	DDwindow (DDinfo ddi) {
		super("DDecon Analysis of "+ddi.lsi.ofi.fileName);
		this.ddi = ddi;
		this.manager = ddi.manager;
		
		dd = new DDProcessor(ddi,this);

		int bd = 10;
		int wTF = dd.cans[0].getWidth();
		int wInfo = 150;
		int yInfo = wTF+2*bd;
		int wPF = dd.cans[2].getWidth();
		int yPF = yInfo + wInfo;
		int yt = yPF + (wPF+2*bd);
		setLayout(null);
		setSize(yt,700);
		panMaster = makeMasterPanel(ddi.zdPts);
		setForeground(SystemColor.controlText);
		setBackground(SystemColor.control);
		panMaster.setBounds(0,0,yt,700);
		add(panMaster);
		initialize();
		show();
		complete = true;
		WindowManager.addWindow(this);
	}
*/	
/*	private void initialize() {
		chModel.select(ddi.cModel.getTitle());
		lfImage.setText(ddi.lsi.ofi.fileName);
		if (ddi.manager == null) ddi.lsi.linescanID = Prefs.get("LSDDecon.linescanID",ddi.lsi.linescanID);
		lfPFID.setText(ddi.lsi.linescanID);
		if (manager.catImageInfo != null) {
			lfSample.setText(ddi.sample);
			lfFluor.setText(ddi.fluor);
		}
		else {
			lfSample.setText(Prefs.get("LSDDecon.sample",""));
			lfFluor.setText(Prefs.get("LSDDecon.fluor",""));
		}
		lfDate.setText(Prefs.get("LSDDecon.date",""));
		lfExpT.setText(Prefs.get("LSDDecon.exposure",""));
		
		boolean[] pStates = ddi.cModel.getParamStates();
		for (int i=0; i<ddi.cModel.getParamNumber(); i++) {
			cbModelStates[i].setState(pStates[i]);
		}
	}
*/	
	public void makeWindow() {
		
		// Instantiate the window components
		panMaster = new Panel();
			panTFplot = new Panel();
			Panel b = new Panel();
//			panTFparams = new Panel();
//				Label[] labModel = new Label [ddp.rm.model.getParamNumber()];
//				tfModelParams = new TextField [ddp.rm.model.getParamNumber()];
//				cbModelStates = new Checkbox [ddp.rm.model.getParamNumber()];
				
//				String[] pNames = ddp.rm.model.getParamNames();
//				double[] params = ddp.rm.model.getParams();
//				boolean[] pStates = ddp.rm.model.getParamStates();
//				for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
//					labModel[i] = new Label(pNames[i]);
//					tfModelParams[i] = new TextField(IJ.d2s(params[i],2),3);
//					cbModelStates[i] = new Checkbox("",pStates[i]);
//				}
			Panel d = new Panel();
				lfImage = new LField("Image",LField.WEST);
				lfPFID = new LField("Linescan ID",LField.WEST);
				lfDate = new LField("Date",LField.WEST);
				lfSample = new LField("Sample",LField.WEST);
				lfFluor = new LField("Fluor",LField.WEST);
				lfExpT = new LField("Exp. Time",LField.WEST);
				chModel = new Choice();
				for (int i=0;i<RepeatModel.Models.length;i++) {
					chModel.add(RepeatModel.Models[i].getTitle());
				}
				Label labDDInfo = new Label("DDecon Information");
				taInfo = new TextArea();
				labPerArea = new Label("");
				labCorrel = new Label("");
				labRsq = new Label("");
				labError = new CLabel("");
				butNormArea = new Button("Norm. Area");
				butFitPF = new Button("Fit Profile");
				butExport = new Button("Export Data");
//				butViewer = new Button("View");
			panProf = new Panel();
			Panel f = new Panel();
//			panZparams = new Panel();
//				Label labNumZ = new Label("# of Z-lines:");
//				tfNumZ = new TextField(IJ.d2s(ddp.rm.repts.length,0),3);
//				cbfxSL = new Checkbox("Fix SL",false);
//				Label labAllP = new Label("All");
//				Label labFxZP = new Label("Fix Positions");
//				cbfxZp1 = new Checkbox("",false);
//				cbfxRW = new Checkbox("Fix RW",false);
//				Label labAllW = new Label("All");
//				Label labFxZW = new Label("Fix Weights");
//				cbfxZw1 = new Checkbox("",false);
//				ddp.rm.makePanels(panZparams, this, this);

				
		// Set the panel layouts
		this.setLayout(null);
			panMaster.setLayout(null);
				panTFplot.setLayout(null);
				b.setLayout(null);
//				panTFparams.setLayout(null);
				d.setLayout(null);
				panProf.setLayout(null);
				f.setLayout(null);
//				panZparams.setLayout(null);

				
		// Set the panel colors
		this.setForeground(SystemColor.controlText);
		this.setBackground(SystemColor.control);
			panMaster.setBackground(Color.gray);

		
		// Set the text fonts
		Font ft = new Font("SanSerif",Font.BOLD,14);
		labPerArea.setFont(ft);
		labCorrel.setFont(ft);
		labRsq.setFont(ft);
		labError.setFont(ft);
		
		
		// Add the action listeners
//		for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
//			tfModelParams[i].addActionListener(this);
//		}
		chModel.addItemListener(this);
		butNormArea.addActionListener(this);
		butFitPF.addActionListener(this);
		butExport.addActionListener(this);
//		butViewer.addActionListener(this);
		MouseListener[] ml = cans[2].getMouseListeners();
		cans[2].removeMouseListener(ml[0]);
		cans[2].addMouseListener(this);
		cans[2].addMouseMotionListener(this);
//		tfNumZ.addActionListener(this);
//		cbfxSL.addItemListener(this);
//		cbfxZp1.addItemListener(this);
//		cbfxRW.addItemListener(this);
//		cbfxZw1.addItemListener(this);
		
		
		// Set the component bounds
		int bd = 10;
		int wTF = cans[0].getWidth();
		int wTF2 = cans[1].getWidth();
		int wInfo = 150;
		int yInfo = wTF+2*bd;
		int wPF = cans[2].getWidth();
		int wPF2 = cans[3].getWidth();
		int yPF = yInfo + wInfo;
//		int yt = wTF+wInfo+wPF+40;
//		this.setSize(yt,700);
//			panMaster.setBounds(0,0,yt,700);
				panTFplot.setBounds(0,20,wTF+2*bd,280);
					cans[0].setBounds(bd,20+bd,wTF,240);
				b.setBounds(0,300,wTF+2*bd,280);
					cans[1].setBounds(bd,bd,wTF2,240);
//				panTFparams.setBounds(0,580,240,100);
//					for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
//						labModel[i].setBounds(20,20*i,80,20);
//						tfModelParams[i].setBounds(100,20*i,80,20);
//						cbModelStates[i].setBounds(180,20*i,20,20);
//					}
				d.setBounds(yInfo,20,wInfo,560);
					lfImage.setBounds(5,30,140,20,0.7);
					lfPFID.setBounds(5,55,140,20,0.6);
					lfDate.setBounds(5,80,140,20,0.7);
					lfSample.setBounds(5,105,140,20,0.6);
					lfFluor.setBounds(5,130,140,20,0.7);
					lfExpT.setBounds(5,155,140,20,0.3);
					chModel.setBounds(5,180,140,20);
					labDDInfo.setBounds(5,205,140,20);
					taInfo.setBounds(5,225,140,50);
					labPerArea.setBounds(20,285,125,20);
					labCorrel.setBounds(20,310,125,20);
					labRsq.setBounds(20,335,125,20);
					labError.setBounds(20,360,125,20);
					butNormArea.setBounds(25,390,100,30);
					butFitPF.setBounds(25,425,100,30);
					butExport.setBounds(25,460,100,30);
//					butViewer.setBounds(25,495,100,30);
				panProf.setBounds(yPF,20,wPF+2*bd,280);
					cans[2].setBounds(bd,20+bd,wPF,240);
				f.setBounds(yPF,300,wPF+2*bd,280);
					cans[3].setBounds(bd,bd,wPF2,240);
//				panZparams.setBounds(240,580,140+70*ddp.rm.repts.length,100);
//					labNumZ.setBounds(0,0,70,20);
//					tfNumZ.setBounds(80,0,30,20);
//					cbfxSL.setBounds(0,20,70,20);
//					labAllP.setBounds(80,20,30,20);
//					labFxZP.setBounds(0,40,70,20);
//					cbfxZp1.setBounds(90,40,30,20);
//					cbfxRW.setBounds(0,60,70,20);
//					labAllP.setBounds(80,60,30,20);
//					labFxZW.setBounds(0,80,70,20);
//					cbfxZw1.setBounds(90,80,30,20);
		
		
		// Add all the components together
		this.add(panMaster);
			panMaster.add(panTFplot);
				panTFplot.add(cans[0]);
			panMaster.add(b);
				b.add(cans[1]);
//			panMaster.add(panTFparams);
//				for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
//					panTFparams.add(labModel[i]);
//					panTFparams.add(tfModelParams[i]);
//					panTFparams.add(cbModelStates[i]);
//				}
			panMaster.add(d);
				d.add(lfImage);	d.add(lfImage.label);
				d.add(lfPFID); d.add(lfPFID.label);
				d.add(lfDate); d.add(lfDate.label);
				d.add(lfSample); d.add(lfSample.label);
				d.add(lfFluor); d.add(lfFluor.label);
				d.add(lfExpT); d.add(lfExpT.label);
				d.add(chModel); 
				d.add(labDDInfo);
				d.add(taInfo);
				d.add(labPerArea);
				d.add(labCorrel);
				d.add(labRsq);
				d.add(labError);
				d.add(butNormArea);
				d.add(butFitPF);
				d.add(butExport);
//				d.add(butViewer);
			panMaster.add(panProf);
				panProf.add(cans[2]);
			panMaster.add(f);
				f.add(cans[3]);
//			panMaster.add(panZparams);
//				panZparams.add(labNumZ);
//				panZparams.add(tfNumZ);
//				panZparams.add(cbfxSL);
//				panZparams.add(labAllP);
//				panZparams.add(labFxZP);
//				panZparams.add(cbfxZp1);
//				panZparams.add(cbfxRW);
//				panZparams.add(labAllW);
//				panZparams.add(labFxZW);
//				panZparams.add(cbfxZw1);
				
		updateModelPanel(false);		
		updateReptPanel(false);
		updateStats2();
	}
	
	public void updateModelPanel(boolean isRemoveNeeded) {
		if (isRemoveNeeded) panMaster.remove(panTFparams);
		panTFparams = new Panel();
		panTFparams.setLayout(null);
		int num = ddp.rm.model.getParamNumber();
		String[] pNames = ddp.rm.model.getParamNames();
		double[] params = ddp.rm.model.getParams();
		boolean[] pStates = ddp.rm.model.getParamStates();
		
		Label[] labModel = new Label [num];
		tfModelParams = new TextField [num];
		cbModelStates = new Checkbox [num];
		
		for (int i=0; i<num; i++) {
			labModel[i] = new Label(pNames[i]);
			labModel[i].setBounds(20,20*i,80,20);
			panTFparams.add(labModel[i]);
			tfModelParams[i] = new TextField(IJ.d2s(params[i],2),3);
			tfModelParams[i].setBounds(100,20*i,80,20);
			tfModelParams[i].addActionListener(this);
			panTFparams.add(tfModelParams[i]);
			cbModelStates[i] = new Checkbox("",pStates[i]);
			cbModelStates[i].setBounds(180,20*i,20,20);
			cbModelStates[i].addItemListener(this);
			panTFparams.add(cbModelStates[i]);
		}
		panTFparams.setBounds(0,580,240,100);
		panMaster.add(panTFparams);
		panMaster.repaint();
	}
	
	public void updateReptPanel(boolean isRemoveNeeded) {
		if (isRemoveNeeded) panMaster.remove(panZparams);
		panZparams = new Panel();
		int nz = ddp.rm.repts.length;
		panZparams.setLayout(null);

		// Row 1
		Label labNumZ = new Label("# of Z-lines:");
		labNumZ.setBounds(0,0,70,20);
		panZparams.add(labNumZ); // (1,1)
		
		String tNumZ = IJ.d2s(nz,0);
		tfNumZ = new TextField(tNumZ,3);
		tfNumZ.setBounds(80,0,30,20);
		tfNumZ.addActionListener(this);
		panZparams.add(tfNumZ); // (1,2)

		// Row 2
		cbfxSL = new Checkbox("Fix SL",false);
		cbfxSL.setBounds(0,20,70,20);
		cbfxSL.addItemListener(this);
		panZparams.add(cbfxSL); // (2,1)
		
		Label labAllP = new Label("All");
		labAllP.setBounds(80,20,30,20);
		panZparams.add(labAllP); // (2,2)

		// Row 3
		Label labFxZP = new Label("Fix Positions");
		labFxZP.setBounds(0,40,70,20);
		panZparams.add(labFxZP); // (3,1)

		cbfxZp1 = new Checkbox("",false);
		cbfxZp1.setBounds(90,40,30,20);
		cbfxZp1.addItemListener(this);
		panZparams.add(cbfxZp1); // (3,2)

		// Row 4
		cbfxRW = new Checkbox("Fix RW",false);
		cbfxRW.setBounds(0,60,70,20);
		cbfxRW.addItemListener(this);
		panZparams.add(cbfxRW); // (4,1)
		
		Label labAllW = new Label("All");
		labAllP.setBounds(80,60,30,20);
		panZparams.add(labAllW); // (4,2)
		
		// Row 5
		Label labFxZW = new Label("Fix Weights");
		labFxZW.setBounds(0,80,70,20);
		panZparams.add(labFxZW); // (5,1)
		
		cbfxZw1 = new Checkbox("",false);
		cbfxZw1.setBounds(90,80,30,20);
		cbfxZw1.addItemListener(this);
		panZparams.add(cbfxZw1); // (5,2)
		
		
		// Repeats
		ddp.rm.makePanels(panZparams, this, this);
		panZparams.setBounds(240,580,140+70*nz,100);

		int w1 = panZparams.getX() + panZparams.getWidth();
		int wt = Math.max(pInfowidth+panProf.getWidth()+panTFplot.getWidth()+40,w1);
		this.setSize(wt,700);
		panMaster.setBounds(0,0,wt,700);
		panMaster.add(panZparams);
		repaint();
	}
	
/*	private Panel makeMasterPanel(Point2D.Double[] pts) {
		int bd = 10;
		int wTF = dd.cans[0].getWidth();
		int wInfo = 150;
		int yInfo = wTF+2*bd;
		int wPF = dd.cans[2].getWidth();
		int yPF = yInfo + wInfo;
		Panel p = new Panel();
		p.setLayout(null);
		p.setBackground(Color.gray);
		p.setSize(1230,700);
		panTFplot = makeTF_plotPanel();
		Panel b = makeTF_resPanel();
		panTFparams = makeTF_contPanel();
		Panel d = makeInfoPanel();
		panProf = makePF_plotPanel();
		Panel f = makePF_resPanel();
		panZparams = makePF_contPanel(pts);
		panTFplot.setBounds(0,20,wTF+2*bd,280);
		b.setBounds(0,300,wTF+2*bd,280);
		panTFparams.setBounds(0,580,240,100);
		d.setBounds(yInfo,20,wInfo,560);
		panProf.setBounds(yPF,20,wPF+2*bd,280);
		f.setBounds(yPF,300,wPF+2*bd,280);
		p.add(panTFplot);
		p.add(b);
		p.add(panTFparams);
		p.add(d);
		p.add(panProf);
		p.add(f);
		p.add(panZparams);
		return p;
	}
*/	
/*	private Panel makeInfoPanel() {
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(200,800);
		int x = 5;
		int y = 30;
		int w = pInfowidth - 2*x;
		int h = 20;
		lfImage = new LField("Image",LField.WEST);
		lfImage.setBounds(x,y,w,h,0.7);
		p.add(lfImage); p.add(lfImage.label);

		lfPFID = new LField("Linescan ID",LField.WEST);
		lfPFID.setBounds(x,y+=25,w,h,0.6);
		p.add(lfPFID); p.add(lfPFID.label);

		lfDate = new LField("Date",LField.WEST);
		lfDate.setBounds(x,y+=25,w,h,0.7);
		p.add(lfDate); p.add(lfDate.label);

		lfSample = new LField("Sample",LField.WEST);
		lfSample.setBounds(x,y+=25,w,h,0.6);
		p.add(lfSample); p.add(lfSample.label);

		lfFluor = new LField("Fluor",LField.WEST);
		lfFluor.setBounds(x,y+=25,w,h,0.7);
		p.add(lfFluor); p.add(lfFluor.label);

		lfExpT = new LField("Exp. Time",LField.WEST);
		lfExpT.setBounds(x,y+=25,w,h,0.3);
		p.add(lfExpT); p.add(lfExpT.label);

		chModel = new Choice();
		for (int i=0;i<DDinfo.Models.length;i++)
			chModel.add(DDinfo.Models[i].getTitle());
		chModel.setBounds(x,y+=25,w,h);
		chModel.addItemListener(this);
		p.add(chModel);
		Label labDDInfo = new Label("DDecon Information");
		labDDInfo.setBounds(x,y+=25,w,h);
		p.add(labDDInfo);
		taInfo = new TextArea();
		taInfo.setBounds(x,y+=20,w,50);
		p.add(taInfo);
		Font f = new Font("SanSerif",Font.BOLD,14);
		int off = 15;
		labPerArea = new Label("Area = "+IJ.d2s(dd.getPerArea(),2)+" %");
		labPerArea.setFont(f);
		labPerArea.setBounds(x+off,y+=60,w-off,h);
		p.add(labPerArea);
		double corr = dd.getCorrel();
		labCorrel = new Label("Corr = "+IJ.d2s(corr,4));
		labCorrel.setFont(f);
		labCorrel.setBounds(x+off,y+=25,w-off,h);
		p.add(labCorrel);
		labRsq = new Label("R-sq = "+IJ.d2s(corr*corr,4));
		labRsq.setFont(f);
		labRsq.setBounds(x+off,y+=25,w-off,h);
		p.add(labRsq);
		labError = new CLabel("Error = "+IJ.d2s(dd.getError(),2)+" %");
		labError.setFont(f);
		labError.setBounds(x+off,y+=25,w-off,h);
		p.add(labError);
		butNormArea = new Button("Norm. Area");
		h=30;
		int bwd = 20;
		butNormArea.setBounds(x+bwd,y+=30,w-2*bwd,h);
		butNormArea.addActionListener(this);
		p.add(butNormArea);
		butFitPF = new Button("Fit Profile");
		butFitPF.setBounds(x+bwd,y+=35,w-2*bwd,h);
		butFitPF.addActionListener(this);
		p.add(butFitPF);
		butExport = new Button("Export");
		butExport.setBounds(x+bwd,y+=35,w-2*bwd,h);
		butExport.addActionListener(this);
		p.add(butExport);
		butViewer = new Button("View");
		butViewer.setBounds(x+bwd,y+=35,w-2*bwd,h);
		butViewer.addActionListener(this);
		p.add(butViewer);
		return p;
	}
*/	
/*	private Panel makeTF_plotPanel() {
		int wTF = dd.cans[0].getWidth();
		int bd = 10;
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(wTF+2*bd,300);
		p.setSize(600,300);
		dd.cans[0].setBounds(bd,20+bd,wTF,240);
		p.add(dd.cans[0]);
		return p;
	}
*/	
/*	private Panel makeTF_resPanel() {
		int wTF = dd.cans[1].getWidth();
		int bd = 10;
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(wTF+2*bd,300);
		dd.cans[1].setBounds(bd,bd,wTF,240);
		p.add(dd.cans[1]);
		return p;
	}
*/	
/*	private Panel makeTF_contPanel() {
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(600,100);
		int num = ddi.cModel.getParamNumber();
		String[] pNames = ddi.cModel.getParamNames();
		double[] params = ddi.cModel.getParams();
		boolean[] pStates = ddi.cModel.getParamStates();
		
		Label[] labModel = new Label [num];
		tfModelParams = new TextField [num];
		cbModelStates = new Checkbox [num];
		
		for (int i=0; i<num; i++) {
			labModel[i] = new Label(pNames[i]);
			labModel[i].setBounds(20,20*i,80,20);
			p.add(labModel[i]);
			tfModelParams[i] = new TextField(IJ.d2s(params[i],2),3);
			tfModelParams[i].setBounds(100,20*i,80,20);
			tfModelParams[i].addActionListener(this);
			p.add(tfModelParams[i]);
			cbModelStates[i] = new Checkbox("",pStates[i]);
			cbModelStates[i].setBounds(180,20*i,20,20);
			p.add(cbModelStates[i]);
		}
		return p;
	}
*/	
/*	private Panel makePF_plotPanel() {
		int wPF = dd.cans[2].getWidth();
		int bd = 10;
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(wPF+2*bd,300);
		dd.cans[2].setBounds(bd,20+bd,wPF,240);
		MouseListener[] ml = dd.cans[2].getMouseListeners();
		dd.cans[2].removeMouseListener(ml[0]);
		dd.cans[2].addMouseListener(this);
		dd.cans[2].addMouseMotionListener(this);
		p.add(dd.cans[2]);
		return p;
	}
*/	
/*	private Panel makePF_resPanel() {
		int wPF = dd.cans[3].getWidth();
		int bd = 10;
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(wPF+2*bd,300);
		dd.cans[3].setBounds(bd,bd,wPF,240);
		p.add(dd.cans[3]);
		return p;
	}
*/	
	private Panel makePF_contPanel(Point2D.Double[] pts) {
		Panel p = new Panel();
		int nz = pts.length;
		p.setLayout(null);
		p.setSize(140+70*nz,100);

		// Row 1
		Label labNumZ = new Label("# of Z-lines:");
		labNumZ.setBounds(0,0,70,20);
		p.add(labNumZ); // (1,1)
		String tNumZ = IJ.d2s(nz,0);
		tfNumZ = new TextField(tNumZ,3);
		tfNumZ.setBounds(80,0,30,20);
		tfNumZ.addActionListener(this);
		p.add(tfNumZ); // (1,2)
		for (int i=0; i<nz; i++) {
			Label labZ = new Label("Z"+(i+1));
			labZ.setBounds(160+70*i,0,30,20);
			p.add(labZ); // (1,3 to nz+2)
		}

		// Row 2
		cbfxSL = new Checkbox("Fix SL",false);
		cbfxSL.setBounds(0,20,70,20);
		cbfxSL.addItemListener(this);
		p.add(cbfxSL); // (2,1)
		Label labAllP = new Label("All");
		labAllP.setBounds(80,20,30,20);
		p.add(labAllP); // (2,2)
		tfZp = new TextField [nz];
		for (int i=0; i<nz; i++) {
			String tzp = IJ.d2s(pts[i].x,2);
			tfZp[i] = new TextField(tzp,3);
			tfZp[i].setBounds(140+70*i,20,60,20);
			tfZp[i].addActionListener(this);
			p.add(tfZp[i]); // (2,3 to nz+2)
		}
		// Row 3
		Label labFxZP = new Label("Fix Positions");
		labFxZP.setBounds(0,40,70,20);
		p.add(labFxZP); // (3,1)
		cbfxZp = new Checkbox [nz+1];
		for (int i=0; i<nz+1; i++) {
			cbfxZp[i] = new Checkbox("",false);
			cbfxZp[i].setBounds(90+70*i,40,30,20);
			cbfxZp[i].addItemListener(this);
			p.add(cbfxZp[i]); // (3,2 to nz+2)
		}
		// Row 4
		cbfxRW = new Checkbox("Fix RW",false);
		cbfxRW.setBounds(0,60,70,20);
		cbfxRW.addItemListener(this);
		p.add(cbfxRW); // (4,1)
		Label labAllW = new Label("All");
		labAllP.setBounds(80,60,30,20);
		p.add(labAllW); // (4,2)
		tfZw = new TextField [nz];
		for (int i=0; i<nz; i++) {
			String tzw = IJ.d2s(pts[i].y,2);
			tfZw[i] = new TextField(tzw,3);
			tfZw[i].setBounds(140+70*i,60,60,20);
			tfZw[i].addActionListener(this);
			p.add(tfZw[i]); // (4,3 to nz+2)
		}
		// Row 5
		Label labFxZW = new Label("Fix Weights");
		labFxZW.setBounds(0,80,70,20);
		p.add(labFxZW); // (5,1)
		cbfxZw = new Checkbox [nz+1];
		for (int i=0; i<nz+1; i++) {
			cbfxZw[i] = new Checkbox("",false);
			cbfxZw[i].setBounds(90+70*i,80,30,20);
			cbfxZw[i].addItemListener(this);
			p.add(cbfxZw[i]); // (5,2 to nz+2)
		}
		p.setBounds(240,580,140+70*nz,100);
		return p;
	}
	
	public void updateZPanel(Point2D.Double[] pts) {
		panMaster.remove(panZparams);
		panZparams = makePF_contPanel(pts);
		int w1 = panZparams.getX() + panZparams.getWidth();
		int wt = Math.max(pInfowidth+panProf.getWidth()+this.panTFplot.getWidth(),w1);
		int h = this.getHeight();
		this.setSize(wt,h);
		panMaster.setSize(wt,h);
		panMaster.add(panZparams);
		repaint();
	}
	
/*	public void updatePlotsAndParams() {
		for (int i=0; i<dd.cans.length; i++) dd.cans[i].repaint();
		panZparams.repaint();
		updateStats();
	}
*/	
/*	public void updateStats() {
		labPerArea.setText("Area = "+IJ.d2s(dd.getPerArea(),2)+" %");
		double corr = dd.getCorrel();
		labCorrel.setText("Corr = "+IJ.d2s(corr,4));
		labRsq.setText("R-sq = "+IJ.d2s(corr*corr,4));
		double e = dd.getError();
		if (e<DDplotter.ERROR1) labError.setColor(Color.green);
		else if (e<DDplotter.ERROR2) labError.setColor(Color.orange);
		else labError.setColor(Color.red);
		labError.setText("Error = "+IJ.d2s(e,2)+" %");
	}
*/	
	public void updateWindow() {
		for (int i=0; i<cans.length; i++) {
			cans[i].repaint();
		}
		panZparams.repaint();
		updateStats2();
	}
	
	public void updateStats2() {
		labPerArea.setText("Area = "+IJ.d2s(ddp.getPerArea(),2)+" %");
		double corr = ddp.getCorrel();
		labCorrel.setText("Corr = "+IJ.d2s(corr,4));
		labRsq.setText("R-sq = "+IJ.d2s(corr*corr,4));
		double e = ddp.getError();
		if (e<DDplotter.ERROR1) labError.setColor(Color.green);
		else if (e<DDplotter.ERROR2) labError.setColor(Color.orange);
		else labError.setColor(Color.red);
		labError.setText("Error = "+IJ.d2s(e,2)+" %");
	}
	
/*	public void updateTFparams (double[] params) {
		for (int i=0; i<params.length; i++) {
			tfModelParams[i].setText(IJ.d2s(params[i],2));
		}
		ddi.cModel.setParams(params);
	}
*/	
/*	public void updateZd(Point2D.Double[] zdPts) {
		int nz = zdPts.length;
		for (int i=0; i<nz; i++) {
			tfZp[i].setText(IJ.d2s(zdPts[i].x,2));
			tfZw[i].setText(IJ.d2s(zdPts[i].y,2));
		}
		ddi.zdPts = zdPts;
	}
*/	
	/** Returns the state of the checkboxes determining which parameters values are fixed*/
/*	public boolean[][] getParamStates() {
		int nmp = ddi.cModel.getParamNumber();
		int nz = cbfxZp.length-1;
		boolean[][] pStates = new boolean [3][];
		pStates[0] = new boolean [nmp];
		pStates[1] = new boolean [nz];
		pStates[2] = new boolean [nz];
		for (int i=0; i<nmp; i++) {
			pStates[0][i] = cbModelStates[i].getState();
		}
		for (int i=0; i<nz; i++) {
			pStates[1][i] = cbfxZp[i+1].getState();
			pStates[2][i] = cbfxZw[i+1].getState();
		}
		return pStates;
	}
*/	
	public void mouseClicked(MouseEvent m) {
		int modifiers = m.getModifiers();
		Point ptClick = m.getPoint();
		Point ptSel = ddp.realToPlot(ptClick);
		
		// Get the coordinates of the plotted Z lines to see of ptClick was near one of them
		Point2D.Double[] repts = ddp.rm.repts;
		Point[] pt = new Point[ddp.rm.repts.length];
		for (int i=0; i<ddp.rm.repts.length; i++) {
			pt[i]= new Point((int)repts[i].x,(int)repts[i].y);
		}
		Point[] reptsReal = ddp.plotToReal(pt);
		int indexD = -1; // indexD is the index of the Zdist point which may be deleted
		int indexI = 0; // indexI is where a new point would be inserted
		int indexJ = -1; // indexJ is the index of the Zdist point which may be jumped up ro down
		for (int i=0; i<ddp.rm.repts.length; i++) {
			double dist2D = Point.distance(ptClick.x,ptClick.y,reptsReal[i].x,reptsReal[i].y);
			int distX = ptClick.x - reptsReal[i].x;
			if (ptClick.x > reptsReal[i].x) indexI = i+1;
			if (dist2D<5) indexD = i;
			if (Math.abs(distX)<5) indexJ = i;
		}
		boolean altKeyDown = (modifiers&ActionEvent.ALT_MASK)!=0 || IJ.altKeyDown();
		boolean shiftKeyDown = (modifiers&ActionEvent.SHIFT_MASK)!=0 || IJ.shiftKeyDown();
		IJ.setKeyUp(KeyEvent.VK_ALT);
		IJ.setKeyUp(KeyEvent.VK_SHIFT);
		
		// Add, delete, or jump Z lines depending on the modifiers
		if (altKeyDown) {
			// if near a Z distribution point, delete it, otherwise insert one
			int nr = ddp.rm.repts.length;
			Point2D.Double[] newPts;
			boolean[] pStates1;
			boolean[] pStates2;
			if (indexD!=-1) {
				// delete Z distribution point
				nr--;
				newPts = new Point2D.Double [nr];
				pStates1 = new boolean [nr];
				pStates2 = new boolean [nr];
				for (int i=0, j=0; i<nr+1; i++, j++) {
					if (i==indexD) i++;
					newPts[j] = repts[i];
					pStates1[j] = ddp.rm.reptFixStates[0][i];
					pStates2[j] = ddp.rm.reptFixStates[1][i];
				}
			} else {
				// insert a new Z distribution point
				nr++;
				newPts = new Point2D.Double [nr];
				pStates1 = new boolean [nr];
				pStates2 = new boolean [nr];
				newPts[indexI] = new Point2D.Double(ptSel.x,ptSel.y);
				pStates1[indexI] = false;
				pStates2[indexI] = false;
				for (int i=0, j=0; i<nr-1; i++, j++) {
					if (i==indexI) j++;
					newPts[j] = repts[i];
					pStates1[j] = ddp.rm.reptFixStates[0][i];
					pStates2[j] = ddp.rm.reptFixStates[1][i];
				}
				isZdistPtSelected = true;
				selPtInd = indexI;
			}
			IJ.setKeyUp(KeyEvent.VK_ALT);
//			tfNumZ.setText(IJ.d2s(nr,0)); // Change numZ value in window

			// Update values
			ddp.rm.repts = newPts;
			ddp.rm.reptFixStates[0] = pStates1;
			ddp.rm.reptFixStates[1] = pStates2;

			updateReptPanel(true);

//			for (int i=0; i<newPts.length; i++) {
//				ddp.rm.tfReptPositions[i].setText(IJ.d2s(newPts[i].x,2));
//				ddp.rm.tfReptWeights[i].setText(IJ.d2s(newPts[i].y,2));
//			}
//			IJ.setKeyUp(KeyEvent.VK_ALT);
			
			ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
			
			ddp.updatePlots(false);

			updateWindow();
			return;
		} else if ((shiftKeyDown && indexJ!=-1) || indexD!=-1) {
			// if above or below a Zdistribution point move it to ptClick
			ddp.rm.repts[indexJ].setLocation(ptSel.x,ptSel.y);
			isZdistPtSelected = true;
			selPtInd = indexJ;
			
			for (int i=0; i<repts.length; i++) {
				ddp.rm.tfReptPositions[i].setText(IJ.d2s(repts[i].x,2));
				ddp.rm.tfReptWeights[i].setText(IJ.d2s(repts[i].y,2));
			}
			repaint();
			IJ.setKeyUp(KeyEvent.VK_SHIFT);
			
			ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
			ddp.updatePlots(false);
			updateWindow();
		}
	}
	
/*	public void mouseClickedOld(MouseEvent m) {
		int modifiers = m.getModifiers();
		Point ptClick = m.getPoint();
		Point ptSel = dd.realToPlot(ptClick);
		
		// Get the coordinates of the plotted Z lines to see of ptClick was near one of them
		Point2D.Double[] zdPts = ddi.zdPts;
		Point[] ptZd = new Point[ddi.numZ];
		for (int i=0; i<ddi.numZ; i++) ptZd[i]= new Point((int)zdPts[i].x,(int)zdPts[i].y);
		Point[] ptZdReal = dd.plotToReal(ptZd);
		int indexD = -1; // indexD is the index of the Zdist point which may be deleted
		int indexI = 0; // indexI is where a new point would be inserted
		int indexJ = -1; // indexJ is the index of the Zdist point which may be jumped up ro down
		for (int i=0; i<ddi.numZ; i++) {
			double dist2D = Point.distance(ptClick.x,ptClick.y,ptZdReal[i].x,ptZdReal[i].y);
			int distX = ptClick.x - ptZdReal[i].x;
			if (ptClick.x > ptZdReal[i].x) indexI = i+1;
			if (dist2D<5) indexD = i;
			if (Math.abs(distX)<5) indexJ = i;
		}
		boolean altKeyDown = (modifiers&ActionEvent.ALT_MASK)!=0 || IJ.altKeyDown();
		boolean shiftKeyDown = (modifiers&ActionEvent.SHIFT_MASK)!=0 || IJ.shiftKeyDown();
		IJ.setKeyUp(KeyEvent.VK_ALT);
		IJ.setKeyUp(KeyEvent.VK_SHIFT);
		
		// Add, delete, or jump Z lines depending on the modifiers
		if (altKeyDown) {
			// if near a Z distribution point, delete it, otherwise insert one
			int nz = ddi.numZ;
			Point2D.Double[] newPts;
			boolean[] pStates1;
			boolean[] pStates2;
			int j;
			if (indexD!=-1) {
				// delete Z distribution point
				nz--;
				newPts = new Point2D.Double [nz];
				pStates1 = new boolean [nz];
				pStates2 = new boolean [nz];
				for (int i=0; i<nz; i++) {
					j = i+((i<indexD)?0:1);
					newPts[i] = zdPts[j];
					pStates1[i] = ddi.pStates[1][j];
					pStates2[i] = ddi.pStates[2][j];
				}
			} else {
				// insert a new Z distribution point
				nz++;
				newPts = new Point2D.Double [nz];
				pStates1 = new boolean [nz];
				pStates2 = new boolean [nz];
				newPts[indexI] = new Point2D.Double(ptSel.x,ptSel.y);
				pStates1[indexI] = false;
				pStates2[indexI] = false;
				for (int i=0; i<nz; i++) {
					if (i==indexI) i++;
					j = i-((i<indexI)?0:1);
					newPts[i] = zdPts[j];
					pStates1[i] = ddi.pStates[1][j];
					pStates2[i] = ddi.pStates[2][j];
				}
				isZdistPtSelected = true;
				selPtInd = indexI;
			}
			tfNumZ.setText(IJ.d2s(nz,0)); // Change numZ value in window

			// Update values
			ddi.numZ = nz;
			ddi.zdPts = newPts;
			ddi.pStates[1] = pStates1;
			ddi.pStates[2] = pStates2;

			updateZPanel(newPts);
			updateZd(newPts); // Updates the text fields in the window and the processor values
			repaint();
			IJ.setKeyUp(KeyEvent.VK_ALT);
			dd.updateAtZdist();
			return;
		} else if ((shiftKeyDown && indexJ!=-1) || indexD!=-1) {
			// if above or below a Zdistribution point move it to ptClick
			ddi.zdPts[indexJ].setLocation(ptSel.x,ptSel.y);
			isZdistPtSelected = true;
			selPtInd = indexJ;
			
			updateZd(zdPts);
			repaint();
			IJ.setKeyUp(KeyEvent.VK_SHIFT);
			dd.updateAtZdist();
		}
	}
*/	
	public void mouseDragged (MouseEvent m) {
		Point ptDrag = m.getPoint();
		Point ptSel = ddp.realToPlot(ptDrag);
		int index;
		if (isZdistPtSelected && selPtInd != -1) {
			// selected point is continuously moved to where the cursor is
			index = selPtInd;
			ddp.rm.repts[index].x = ptSel.x;
			ddp.rm.repts[index].y = ptSel.y;
			ddp.rm.tfReptPositions[index].setText(IJ.d2s(ptSel.x,2));
			ddp.rm.tfReptWeights[index].setText(IJ.d2s(ptSel.y,2));
			repaint();

			
			ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
			
			ddp.updatePlots(false);

			updateWindow();
		} else {
			// if a point in the Z line distribution is close to the ptClick, move it
			Point[] ptZd = new Point[ddp.rm.repts.length];
			for (int i=0; i<ddp.rm.repts.length; i++) {
				ptZd[i]= new Point((int)ddp.rm.repts[i].x,(int)ddp.rm.repts[i].y);
			}
			Point[] ptZdReal = ddp.plotToReal(ptZd);

			index=-1;
			double dist;
			for (int i=0; i<ddp.rm.repts.length; i++) {
				dist = Point.distance(ptDrag.x,ptDrag.y,ptZdReal[i].x,ptZdReal[i].y);
				if (dist<10) index = i;
			}

			if (index!=-1) {
				ddp.rm.repts[index].x = ptSel.x;
				ddp.rm.repts[index].y = ptSel.y;
				ddp.rm.tfReptPositions[index].setText(IJ.d2s(ptSel.x,2));
				ddp.rm.tfReptWeights[index].setText(IJ.d2s(ptSel.y,2));
				isZdistPtSelected = true;
				selPtInd = index;
				repaint();

				ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
				
				ddp.updatePlots(false);

				updateWindow();
			}
		}
	}
	
/*	public void mouseDraggedOld (MouseEvent m) {
		Point ptDrag = m.getPoint();
		Point ptSel = dd.realToPlot(ptDrag);
		int index;
		if (isZdistPtSelected && selPtInd != -1) {
			// selected point is continuously moved to where the cursor is
			index = selPtInd;
			ddi.zdPts[index].x = ptSel.x;
			ddi.zdPts[index].y = ptSel.y;
			tfZp[index].setText(IJ.d2s(ptSel.x,2));
			tfZw[index].setText(IJ.d2s(ptSel.y,2));
			repaint();
			dd.updateAtZdist();
		} else {
			// if a point in the Z line distribution is close to the ptClick, move it
			Point[] ptZd = new Point[ddi.numZ];
			for (int i=0; i<ddi.numZ; i++) ptZd[i]= new Point((int)ddi.zdPts[i].x,(int)ddi.zdPts[i].y);
			Point[] ptZdReal = dd.plotToReal(ptZd);

			index=-1;
			double dist;
			for (int i=0; i<ddi.numZ; i++) {
				dist = Point.distance(ptDrag.x,ptDrag.y,ptZdReal[i].x,ptZdReal[i].y);
				if (dist<10) index = i;
			}

			if (index!=-1) {
				ddi.zdPts[index].x = ptSel.x;
				ddi.zdPts[index].y = ptSel.y;
				tfZp[index].setText(IJ.d2s(ptSel.x,2));
				tfZw[index].setText(IJ.d2s(ptSel.y,2));
				isZdistPtSelected = true;
				selPtInd = index;
				repaint();
				dd.updateAtZdist();
			}
		}
	}
*/	
	public void mouseReleased(MouseEvent e) {
		isZdistPtSelected = false;
		selPtInd = -1;
	}

	public void mouseMoved (MouseEvent m) {}
	
	public void actionPerformed(ActionEvent a) {
		Object s = a.getSource();
		for (int i=0; i<tfModelParams.length; i++) {
			if (s==tfModelParams[i]) {
				doChangeModelParams2();
//				doChangeModelParams();
				return;
			}
		}
		for (int i=0; i<ddp.rm.repts.length; i++) {
			if (s==ddp.rm.tfReptPositions[i]) {
				doChangeZPosition2(i);
//		for (int i=0; i<ddi.numZ; i++) {
//			if (s==tfZp[i]) {
//				doChangeZPosition(i);
				return;
			}
			if (s==ddp.rm.tfReptWeights[i]) {
				doChangeZWeight2(i);
//		for (int i=0; i<ddi.numZ; i++) {
//			if (s==tfZw[i]) {
//				doChangeZWeight(i);
				return;
			}
		}
		if (s==butNormArea) {
			doNormalize2();
//			doNormalize();
		}
		else if (s==butFitPF) {
			ddp.manager.doFitting();
//			doFitting2();
		}
		else if (s==butExport) {
			ddp.manager.exportData();
//			doExport();
		}
//		else if (s==butViewer) {
//			doSarcView();
//		}
		else if (s==tfNumZ) {
			doChangeNumZ2();
//			doChangeNumZ();
		}
	}
	
	public void doChangeModelParams2() {
		double[] modParams = new double [tfModelParams.length];
		for (int i=0; i<tfModelParams.length; i++) {
			modParams[i] = Double.parseDouble(tfModelParams[i].getText());
		}
		ddp.rm.model.setParams(modParams);
//		ddp.rm.psf.focusParam = modParams[0];
		ddp.rm.psf.makePixels(ddp.rm.psf.pixels.length);
		ddp.rm.modelProfilePixels = ddp.rm.calcModelInts(ddp.rm.model.getParams());
		ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
		
		ddp.updatePlots(true);

		updateWindow();
	}
	
/*	public void doChangeModelParams() {
		double[] modParams = new double [tfModelParams.length];
		for (int j=0; j<tfModelParams.length; j++) {
			modParams[j] = Double.parseDouble(tfModelParams[j].getText());
		}
		ddi.cModel.setParams(modParams);
		dd.updateAtPSF();
	}
*/	
	public void doChangeZPosition2(int r) {
		ddp.rm.repts[r].x = Double.parseDouble(ddp.rm.tfReptPositions[r].getText());

		ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
		
		ddp.updatePlots(false);

		updateWindow();
	}
	
	public void doChangeZWeight2(int r) {
		ddp.rm.repts[r].y = Double.parseDouble(ddp.rm.tfReptWeights[r].getText());

		ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
		
		ddp.updatePlots(false);

		updateWindow();
	}
	
/*	public void doChangeZPosition(int i) {
		ddi.zdPts[i].x = Double.parseDouble(tfZp[i].getText());
		dd.updateAtZdist();
	}
*/	
/*	public void doChangeZWeight(int i) {
		ddi.zdPts[i].y = Double.parseDouble(tfZw[i].getText());
		dd.updateAtZdist();
	}
*/	
	public void doNormalize2() {
		double scale = 100/ddp.getPerArea();
		for (int i=0; i<ddp.rm.repts.length; i++) ddp.rm.repts[i].y *= scale;
		for (int i=0; i<ddp.rm.repts.length; i++) {
			ddp.rm.tfReptWeights[i].setText(IJ.d2s(ddp.rm.repts[i].y,2));
		}
		repaint();
		ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
		
		ddp.updatePlots(false);

		updateWindow();
	}
	
/*	public void doNormalize() {
		double scale = 100/dd.getPerArea();
		for (int i=0; i<ddi.numZ; i++) ddi.zdPts[i].y *= scale;
		for (int i=0; i<ddi.numZ; i++) {
//			tfZp[i].setText(IJ.d2s(ddi.zdPts[i].x,2));
			tfZw[i].setText(IJ.d2s(ddi.zdPts[i].y,2));
		}
		repaint();
		dd.updateAtZdist();
	}
*/	
/*	public boolean doFitting2() {
		IJ.showStatus("Fitting Profile....");
		
		
			boolean[][] pStates = new boolean [3][];
			pStates[0] = new boolean [ddp.rm.model.getParamNumber()];
			for (int i=0; i<ddp.rm.model.getParamNumber(); i++) {
				pStates[0][i] = cbModelStates[i].getState();
			}
			pStates[1] = new boolean [ddp.rm.repts.length];
			pStates[2] = new boolean [ddp.rm.repts.length];
			for (int i=0; i<ddp.rm.repts.length; i++) {
				pStates[1][i] = ddp.rm.cbReptPositions[i].getState();
				pStates[2][i] = ddp.rm.cbReptWeights[i].getState();
			}
//			ddi.pStates = pStates;
//		ddi.pStates = getParamStates(); // set the current param fix states in ddecon processor

		fitter = new DDCurveFitter(ddp.rm, ddp.ls.lsInts);
//		fitter = new DDCurveFitter(dd);
		boolean fit = fitter.doFit(true);
		if (fit) {
			IJ.showStatus("Fit Completed");
			IJ.beep();

			// Update the plots and display the results of the fit
			double[] nparams = fitter.decodeParams(fitter.getParams());
			
			int nmp = ddp.rm.model.getParamNumber();
			double[] mParams = new double [nmp];
			for (int i=0; i<nmp; i++) {
				tfModelParams[i].setText(IJ.d2s(nparams[i],2));
				mParams[i] = nparams[i];
			}
			ddp.rm.psf.focusParam = mParams[0];
			ddp.rm.model.setParams(mParams);
//			for (int i=0; i<nmp; i++) {
//				mParams[i] = nparams[i];
//			}
//			for (int i=0; i<mParams.length; i++) {
//				tfModelParams[i].setText(IJ.d2s(mParams[i],2));
//			}
//			ddp.rm.model.setParams(mParams);
//			updateTFparams(mParams);

			int nr = ddp.rm.repts.length;
			Point2D.Double[] repts = new Point2D.Double [nr];
			for (int i=0; i<nr; i++) {
				repts[i] = new Point2D.Double(nparams[nmp+i],nparams[nmp+nr+i]);
				ddp.rm.tfReptPositions[i].setText(IJ.d2s(repts[i].x,2));
				ddp.rm.tfReptWeights[i].setText(IJ.d2s(repts[i].y,2));
			}
			ddp.rm.repts = repts;
//			updateZd(repts);
			
			
			
			ddp.rm.modelProfilePixels = ddp.rm.calcModelInts(ddp.rm.model.getParams());
			ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);

			ddp.updatePlots(true);

			updateWindow();
		    
//			fitter.setInfo(ddi);

			
			
			
			
			Prefs.set("LSDDecon.gridIntvl",DDplotter.GRID_LINE_INTERVAL); // XXX not user adjustable yet
			Prefs.set("LSDDecon.autoNorm",true); // XXX Not user adjustable yet
			Prefs.set("LSDDecon.pwpp",DDplotter.PLOT_W_PERPIXEL); // XXX Not user adjustable yet
			Prefs.set("LSDDecon.minTFL",fitter.minTFL);
			Prefs.set("LSDDecon.maxTFL",fitter.maxTFL);
//			dd.savePreferences();
			
			
			Prefs.set("LSDDecon.maxOverhang",repeatModel.MAX_OVERHANG); // XXX not user adjustable yet
			Prefs.set("LSDDecon.defModIndex",ddp.rm.cModIndex); // XXX Set to current model used
			Prefs.set("LSDDecon.pSearchR",fitter.max_pos_search);
			Prefs.set("LSDDecon.wSearchR",fitter.max_int_search);
			double aveSL = repeatModel.REPEAT_SPACING;
//			int nr = ddp.rm.repts.length;
			if (nr > 1)
				aveSL = (ddp.rm.repts[nr-1].x - ddp.rm.repts[0].x)/(nr-1); // average sarcomere length
			Prefs.set("LSDDecon.aveSL", aveSL);
			Prefs.set("LSDDecon.linescanID",this.lfPFID.getText());
			Prefs.set("LSDDecon.date",this.lfDate.getText());
			Prefs.set("LSDDecon.sample",this.lfSample.getText());
			Prefs.set("LSDDecon.fluor",this.lfFluor.getText());
			Prefs.set("LSDDecon.exposure",Double.parseDouble(this.lfExpT.getText()));
//			ddi.savePreferences();
		}
		return true;
	}
*/	
/*	public boolean doFitting() {
		IJ.showStatus("Fitting Profile....");
		ddi.pStates = getParamStates(); // set the current param fix states in ddecon processor

		fitter = new DDCurveFitter(dd);
		boolean fit = fitter.doFit(true);
		if (fit) {
			IJ.showStatus("Fit Completed");
			IJ.beep();

			// Update the plots and display the results of the fit
			double[] nparams = dd.decodeParams(fitter.getParams());
			int nmp = ddi.cModel.getParamNumber();
			double[] mParams = new double [nmp];
			for (int i=0; i<nmp; i++) {
				mParams[i] = nparams[i];
			}
			updateTFparams(mParams);

			Point2D.Double[] zdPts = new Point2D.Double [ddi.numZ];
			for (int i=0; i<ddi.numZ; i++) {
				zdPts[i] = new Point2D.Double(nparams[nmp+i],nparams[nmp+ddi.numZ+i]);
			}
			updateZd(zdPts);
			dd.updateAtPSF();
			fitter.setInfo(ddi);
			dd.savePreferences();
			ddi.savePreferences();
		}
		return true;
	}
*/	
/*	public void doExport() {
		ddi.pStates = getParamStates(); // get the current param fix states in ddecon processor
		ddi.lsi.linescanID = lfPFID.getText();
		ddi.date = lfDate.getText();
		ddi.sample = lfSample.getText();
		ddi.fluor = lfFluor.getText();
		try {
			ddi.expTime = Double.parseDouble(lfExpT.getText());
				} catch (Exception NumberFormatException) {ddi.expTime = 0;}
		ddi.model = chModel.getSelectedItem();
		ddi.infoDDecon = taInfo.getText();
		
		
		// Make sure results window is open and set
		if (manager.results == null) {
			String resultsName = "Distributed Deconvolution Results";
			Frame win = WindowManager.getFrame(resultsName);
			if (win!=null){
				// set an open results window from a previous analysis.
				manager.results = (TextWindow)win;
			}
			else {
				String defPath = "";
				manager.resultsSavePath = Prefs.get("LSDDecon.resultsSavePath",defPath);
				// open up lastUsedResults (XXX currently set to null, call Prefs.set somewhere)
				manager.results = manager.openResults(manager.resultsSavePath);
				
				// Do nothing if canceled or file does not open
				if (manager.results==null) {
		    		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		    		int h = 200;
		    		manager.results = new TextWindow(resultsName,"",d.width,h);
		    		manager.results.setLocation(0,d.height-h-30);
		    		manager.results.addWindowListener(manager);
		    	}
			}
		}
		ddi.summarize();
		saveResults(manager.resultsSavePath);
		dd.savePreferences();
		ddi.savePreferences();
	}
*/	
/*	public void saveResults(String path) {
		if (path.equals("")) {
			SaveDialog sd = new SaveDialog("Save as Text", manager.results.getTitle(), ".txt");
			String file = sd.getFileName();
			if (file == null) return;
			path = sd.getDirectory() + file;
		}
		PrintWriter pw = null;
		try {
			FileOutputStream fos = new FileOutputStream(path);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			pw = new PrintWriter(bos);
		}
		catch (IOException e) {
			//IJ.write("" + e);
			return;
		}
		manager.results.getTextPanel().save(pw);
		pw.close();
		IJ.showStatus("");
		Prefs.set("LSDDecon.resultsSavePath",path);
	}
*/	
/*	public void saveIntensities(String path) {
    	String name = "Intensities";
    	String intHeadings = "Position\tI_Obs\tI_Calc\tCalcData";
    	String intValues;
    	TextWindow win = new TextWindow(name,"",300,200);
    	TextPanel tp = win.getTextPanel();
   		tp.setColumnHeadings(" " + intHeadings);

    	for (int i=0; i<ddi.lsi.corrLSint.length; i++) {
       		intValues = i + "\t" + IJ.d2s(ddi.lsi.corrLSint[i],4) + "\t" + IJ.d2s(ddi.zpPix[i].y,4) + "\t";
    		if (i==0) intValues = intValues + this.ddi.lsi.bg1calc + "\t";
    		if (i==1) intValues = intValues + this.ddi.lsi.rawLScalc + "\t";
    		if (i==2) intValues = intValues + this.ddi.lsi.bg2calc + "\t";
    		if (i==3) intValues = intValues + this.ddi.lsi.corrcalc + "\t";
    		tp.appendLine(intValues);
    	}
    	tp.saveAs(path);
    	win.close();
	}
*/	
/*	public boolean doSarcView() {
		IJ.log("Starting sarcView");
		ddi.pStates = getParamStates(); // set the current param fix states in ddecon processor
		if (ddi==null) ddi = new DDinfo();
		SarcData[] sd = dd.prepView();
		
		// Make new ResultWindow with the view results
		double p;
		double q;
		TextWindow tw = new TextWindow("","",100,100);
		WindowManager.addWindow(tw);
		tw.setVisible(false);
		TextPanel tp = tw.getTextPanel();
		String path;
		String headings = "SarcData\tDistance From Z\tDistance From M\tIobs\tIcalc\tIres";
		tp.setColumnHeadings(headings);
		String lineOut;
		for (int n=0; n<sd.length; n++) {
			for (int i=0; i<sd[n].npix; i++) {
				lineOut = " " + "\t";
				p = sd[n].length/2 - sd[n].xVals[i];
				q = sd[n].iVals[0][i]-sd[n].iVals[1][i];
				if (i==0) lineOut = IJ.d2s(n,0) + "\t";
				if (i==1) lineOut = IJ.d2s(sd[n].length,4) + "\t";
				if (i==2) lineOut = IJ.d2s(ddi.zdPts[n].y,4) + "\t";
				if (i==3) lineOut = IJ.d2s(ddi.zdPts[n+1].y,4) + "\t";
				lineOut = lineOut + IJ.d2s(sd[n].xVals[i],4) + "\t"
								+ IJ.d2s(p,4) + "\t" 
								+ IJ.d2s(sd[n].iVals[0][i],4) + "\t" 
								+ IJ.d2s(sd[n].iVals[1][i],4) + "\t"
								+ IJ.d2s(q,4) + "\t";
				tp.appendLine(lineOut);
			}
			String title = "Sarcomere-"+n;
			tw.setTitle(title);
			tw.show();
//			results.show(title);
			path = ddi.lsi.ofi.directory + ddi.lsi.basename + "_sarc"+n+".txt";
			IJ.log("DDwindow.doSarcView: path= "+path);
//			path="";
			PrintWriter pw = null;
			try {
				FileOutputStream fos = new FileOutputStream(path);
				BufferedOutputStream bos = new BufferedOutputStream(fos);
				pw = new PrintWriter(bos);
			}
			catch (IOException e) {
				IJ.write("" + e);
			}
			tp.save(pw);
			pw.close();
			tp.selectAll();
			tp.clearSelection();
			tp.resetSelection();
//			tp.saveAs(path);
//			IJ.runPlugIn("Excel_Writer",path);
//			results.reset();
		}
		tw.close();
		dd.savePreferences();
		ddi.savePreferences();
		return true;
	}
*/	
/*	public void doChangeNumZ() {
		int n = Integer.parseInt(tfNumZ.getText());
		if (n != ddi.numZ) {
			Point2D.Double[] zdPts = new Point2D.Double [n];
			boolean[] pStates1 = new boolean[n];
			boolean[] pStates2 = new boolean[n];
			
			// Fill in the new array with the old values
			double sumZW = 0;
			for (int i=0; i<n; i++) {
				if (i<ddi.numZ) {
					zdPts[i] = ddi.zdPts[i]; 
					pStates1[i] = ddi.pStates[1][i];
					pStates2[i] = ddi.pStates[2][i];
					sumZW += ddi.zdPts[i].y;
				}
				else {
					double x = 2*ddi.zdPts[i-1].x-ddi.zdPts[i-2].x;
					zdPts[i] = new Point2D.Double(x,sumZW/ddi.numZ);
					pStates1[i] = false;
					pStates2[i] = false;
				}
			}
			
			// Autonormalize positions
			double scale = ddi.lsi.lsLength/(zdPts[n-1].x + zdPts[0].x);
			System.out.println("DDwindow.doChangeNumZ: scale="+scale);
			for (int i=0; i<n; i++) {
				zdPts[i].x *= scale;
			}
			
			// Update values
			ddi.numZ = n;
			ddi.zdPts = zdPts;
			ddi.pStates[1] = pStates1;
			ddi.pStates[2] = pStates2;
			updateZPanel(zdPts);
			dd.updateAtZdist();
			doNormalize();
		}
	}
*/	
	public void doChangeNumZ2() {
		int nr = Integer.parseInt(tfNumZ.getText());
		if (nr != ddp.rm.repts.length) {
			Point2D.Double[] repts = new Point2D.Double [nr];
			boolean[] pStates1 = new boolean[nr];
			boolean[] pStates2 = new boolean[nr];
			
			// Fill in the new array with the old values
			double sumZW = 0;
			for (int i=0; i<nr; i++) {
				if (i<ddp.rm.repts.length) {
					repts[i] = ddp.rm.repts[i]; 
					pStates1[i] = ddp.rm.reptFixStates[0][i];
					pStates2[i] = ddp.rm.reptFixStates[1][i];
					sumZW += ddp.rm.repts[i].y;
				}
				else {
					double x = 2*repts[i-1].x-repts[i-2].x;
//					double x = 2*ddp.rm.repts[i-1].x-ddp.rm.repts[i-2].x;
					repts[i] = new Point2D.Double(x,sumZW/ddp.rm.repts.length);
					pStates1[i] = false;
					pStates2[i] = false;
				}
			}
			
			// Autonormalize positions
			double scale = ddp.ls.lsLength/(repts[nr-1].x + repts[0].x);
			for (int i=0; i<nr; i++) {
				repts[i].x *= scale;
			}
			
			// Update values
			ddp.rm.repts = repts;
			ddp.rm.reptFixStates[0] = pStates1;
			ddp.rm.reptFixStates[1] = pStates2;
			
			updateReptPanel(true);
			ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
			ddp.updatePlots(false);
			updateWindow();
			doNormalize2();
		}
	}
	
	public void doChangeModel() {
		ddp.rm.updateModel(chModel.getSelectedIndex());
		
		updateModelPanel(true);
		
		ddp.rm.modelProfilePixels = ddp.rm.calcModelInts(ddp.rm.model.getParams());
		ddp.rm.calcInts = ddp.rm.calcProfileInts(ddp.rm.modelProfilePixels,ddp.rm.repts);
		
		ddp.normalize();
		ddp.updatePlots(true);
		updateWindow();
//		for (int i=0; i<cbModelStates.length; i++) {
//			System.out.println("DDwin.doChangeModel: cbModelStates["+i+"]= "+cbModelStates[i].getState());
//		}
	}
	
	public boolean[] getModelParamStates() {
		boolean[] s = new boolean[cbModelStates.length];
		for (int i=0; i<s.length; i++) s[i] = cbModelStates[i].getState();
		return s;
	}
	
	public void itemStateChanged(ItemEvent e) {
		Object s = e.getSource();
//		System.out.println("DDwin.itemStateChanged: s="+s.toString());
		if (s==chModel) {
			doChangeModel();
			return;
		}
		int mpn = ddp.rm.model.getParamNumber();
		for (int i=0; i<mpn; i++) {
			if (s==cbModelStates[i]) {
				boolean[] curModStates = new boolean[mpn];
				for (int j=0; j<mpn; j++) {
					curModStates[j] = cbModelStates[j].getState();
				}
				ddp.rm.model.setParamStates(curModStates);
//				for (int k=0; k<mpn; k++) {
//					System.out.println("DDwin.itemStateChanged: model.paramState["+k+"]="+ddp.rm.model.getParamStates()[k]);
//				}
			}
		}
		int nz = ddp.rm.repts.length;
		if (s==cbfxZp1) {
			boolean fixAll = cbfxZp1.getState();
			for (int i=0; i<nz; i++) {
				ddp.rm.cbReptPositions[i].setState(fixAll);
				ddp.rm.reptFixStates[0][i] = fixAll;
			}
//			for (int i=1; i<nz+1; i++) ddp.rm.cbReptPositions[i].setState(fixAll);
			return;
		}
		if (s==cbfxZw1) {
			boolean fixAll = cbfxZw1.getState();
			for (int i=0; i<nz; i++) {
				ddp.rm.cbReptWeights[i].setState(fixAll);
				ddp.rm.reptFixStates[1][i] = fixAll;
			}
//			for (int i=1; i<nz+1; i++) ddp.rm.cbReptWeights[i].setState(fixAll);
			return;
		}
		boolean isCheckAllFixed = false;
		for (int i=0; i<nz; i++) {
			if (s==ddp.rm.cbReptPositions[i]) {
				ddp.rm.reptFixStates[0][i] = ddp.rm.cbReptPositions[i].getState();
				if (ddp.rm.cbReptPositions[i].getState()) isCheckAllFixed = true;
				else cbfxZp1.setState(false);
			}
		}
		if (isCheckAllFixed) {
			boolean isAllFixed = true;
			for (int i=0; i<nz; i++) {
				if (!ddp.rm.cbReptPositions[i].getState()) isAllFixed = false;
			}
			if (isAllFixed) cbfxZp1.setState(true);
		}
		isCheckAllFixed = false;
		for (int i=0; i<nz; i++) {
			if (s==ddp.rm.cbReptWeights[i]) {
				ddp.rm.reptFixStates[1][i] = ddp.rm.cbReptWeights[i].getState();
				if (ddp.rm.cbReptWeights[i].getState()) isCheckAllFixed = true;
				else cbfxZw1.setState(false);
			}
		}
		if (isCheckAllFixed) {
			boolean isAllFixed = true;
			for (int i=0; i<nz; i++) {
				if (!ddp.rm.cbReptWeights[i].getState()) isAllFixed = false;
			}
			if (isAllFixed) cbfxZw1.setState(true);
		}
	}
	
/*	public void itemStateChangedOld(ItemEvent e) {
		Object s = e.getSource();
		if (s==chModel) {
			ddi.cModIndex = chModel.getSelectedIndex();
			SModel m = DDinfo.Models[ddi.cModIndex];
			Prefs.set("LSDDecon.defModIndex",ddi.cModIndex);
//			System.out.println("DDwindow.itemStateChanged: cModIndex="+ddi.cModIndex);
			ddi.cModel = m;
			ddi.pStates[0] = m.getParamStates(); // model pStates
			ddi.cModel.setParams(m.getParams());
			
			panMaster.remove(panTFparams);
			panTFparams = makeTF_contPanel();
			panTFparams.setBounds(0,580,240,100);
			panMaster.add(panTFparams);
			panMaster.repaint();
			for (int i=0; i<m.getParamNumber(); i++) {
				cbModelStates[i].setState(ddi.pStates[0][i]);
				tfModelParams[i].setText(IJ.d2s(ddi.cModel.getParams()[i],2));
			}
			dd.autonormalize = true;
			dd.updateAtPSF();
			return;
		}
		int nz = ddi.numZ;
		if (s==cbfxZp[0]) {
			boolean fixAll = cbfxZp[0].getState();
			for (int i=1; i<nz+1; i++) cbfxZp[i].setState(fixAll);
			return;
		}
		if (s==cbfxZw[0]) {
			boolean fixAll = cbfxZw[0].getState();
			for (int i=1; i<nz+1; i++) cbfxZw[i].setState(fixAll);
			return;
		}
		boolean isCheckAllFixed = false;
		for (int i=1; i<nz+1; i++) {
			if (s==cbfxZp[i]) {
				if (cbfxZp[i].getState()) isCheckAllFixed = true;
				else cbfxZp[0].setState(false);
			}
		}
		if (isCheckAllFixed) {
			boolean isAllFixed = true;
			for (int i=1; i<nz+1; i++) {
				if (!cbfxZp[i].getState()) isAllFixed = false;
			}
			if (isAllFixed) cbfxZp[0].setState(true);
		}
		isCheckAllFixed = false;
		for (int i=1; i<nz+1; i++) {
			if (s==cbfxZw[i]) {
				if (cbfxZw[i].getState()) isCheckAllFixed = true;
				else cbfxZw[0].setState(false);
			}
		}
		if (isCheckAllFixed) {
			boolean isAllFixed = true;
			for (int i=1; i<nz+1; i++) {
				if (!cbfxZw[i].getState()) isAllFixed = false;
			}
			if (isAllFixed) cbfxZw[0].setState(true);
		}
	}
*/	
	/** Override Container getInsets() to make room for some text above the image. */
	public Insets getInsets() {
		Insets insets = super.getInsets();
		return new Insets(insets.top+TEXT_GAP, insets.left, insets.bottom, insets.right);
	}

    public void drawInfo(Graphics g) {}
    public void paint(Graphics g) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
}

