/*  DDecon - Distributed Deconvolution Plugin for ImageJ
    Copyright (C) 2007  Ryan Littlefield

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import ij.*;
import ij.ImagePlus;
import ij.gui.ImageCanvas;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

public class LineScanCanvas extends ImageCanvas {
    ImagePlus imp;
    Rectangle [] regions = {};
    boolean overlay = false;
    
    LineScanCanvas(ImagePlus imp) {
        super(imp);
        this.imp = imp;
        this.overlay = false;
    }

    LineScanCanvas(ImagePlus imp, Rectangle[] regions) {
        super(imp);
        this.imp = imp;
        this.regions = regions;
        this.overlay = true;
    }

    public void paint(Graphics g) {
        super.paint(g);
        if (overlay) drawLineScanOverlay(g);
     }
    
/*    public int[] getImageSize() {
    	return new int [] {imageWidth,imageHeight};
    }
*/    
/*    public void setImageSize(int w, int h) {
    	imageWidth = w;
    	imageHeight = h;
    }
*/
	public void updateImage(ImagePlus imp) {
			this.imp = imp;
			int width = imp.getWidth();
			int height = imp.getHeight();
			
			imageWidth = width;
			imageHeight = height;
			srcRect.setFrame(0,0,imageWidth,imageHeight);
			double mag = getMagnification();
			setDrawingSize((int)(imageWidth*mag), (int)(imageHeight*mag));
//			this.setMagnification(1.0);
//	        System.out.println("LSCan.updateImage: draw size="+(int)(imageWidth*mag)+"x"+(int)(imageHeight*mag));
		}
	
    void drawLineScanOverlay(Graphics g) {
        double mag = getMagnification();
        Color[] regionColors = {Color.yellow,Color.red,Color.yellow};
        for (int i=0; i<regions.length; i++) {
        	g.setColor(regionColors[i]);
        	g.drawRect((int)(mag*regions[i].x)+1
        			,(int)(mag*regions[i].y)+1
        			,(int)(mag*regions[i].width)-2
        			,(int)(mag*regions[i].height)-2);
        }
   }

    public void mousePressed(MouseEvent e) {
        super.mousePressed(e);
    }
}

